create procedure sp_dbaman_get_cols @table_name varchar(30)
as 
begin
  select Column_name = c.name,
         Type = t.name,
         Nulls = convert(varchar(8),convert(bit, (c.status & 8))),
         Length = convert(varchar(6),c.length),
         Prec = convert(varchar(6),c.prec),
         c.colid
    into #helptype
    from syscolumns c, systypes t
   where c.id = object_id(@table_name)
     and c.usertype *= t.usertype

  update #helptype
     set Length = ''
   where Type not like '%char%'
     and Type <> 'numeric'

  update #helptype
     set Length = '(' + Length + ')'
   where Type like '%char%'
      or Type = 'char'

  update #helptype
     set Length = '(' + Prec + ')'
   where Type = 'numeric'

  update #helptype
     set Nulls = 'NULL'
   where Nulls = '1'


  update #helptype
     set Nulls = 'NOT NULL'
   where Nulls = '0'

  select column = Column_name 
                  + replicate(' ',32 - char_length(Column_name)) + Type 
                  + Length
                  + replicate(' ',30 - char_length(Type + Length)) + Nulls
    from #helptype
     order by colid
    drop table #helptype
end

go


create procedure sp_dbaman_get_index @objname varchar(92),
                                     @indname varchar(92) = null
as

begin

  declare @indid int			/* the index id of an index */
  declare @keys varchar(200)		/* string to build up index key in */
  declare @inddesc varchar(68)		/* string to build up index desc in */
  declare @inddesc1 varchar(255)		/* string to build up index desc in */
  declare @inddesc2 varchar(50)		/* string to build up index desc in */
  declare @inddesc4 varchar(100)		/* string to build up index desc in */
  declare @inddesc3 varchar(80)		/* string to build up index desc in */
  declare @msg varchar(90)
  declare @len1 int, @len2 int

  declare @i int
  declare @thiskey varchar(30)
  declare @lastindid int

  create table #spindtab
  (
    index_description1  varchar(255),
    index_description2  varchar(60),
    index_description3  varchar(100),
    index_keys          varchar(255)
  )

  if @indname is null
  begin

  select @indid = min(indid)
    from sysindexes
   where id = object_id(@objname)
     and indid > 0
     and indid < 255


  while @indid != NULL
  begin

    select @inddesc1 = 'create '
    select @inddesc2 = ''
    select @inddesc4 = ''
  	
    select @keys = "", @i = 1

    set nocount on

    while @i <= 16
    begin
      select @thiskey = index_col(@objname, @indid, @i)

      if @thiskey = NULL
      begin
        goto keysdone1
      end

      if @i > 1
      begin
        select @keys = @keys + ", "
      end

      select @keys = @keys + index_col(@objname, @indid, @i)

      /*
      **  Increment @i so it will check for the next key.
      */
      select @i = @i + 1

    end


    /*
    **  When we get here we now have all the keys.
    */
    keysdone1:
      set nocount off

    /*
    **  Initialize the index description by figuring out if it's a
    **  clustered or nonclustered index.
    */
    if @indid = 1
    begin
      select @inddesc = "clustered"
      select @inddesc3 = " clustered index "
    end
    if @indid > 1
    begin
      select @inddesc3 = " nonclustered index "
      select @inddesc = "nonclustered"
    end

    /*
    **  Now we'll check out the status bits for this index and
    **  build an english description from them.
    */

    /*
    **  See if the index is unique (0x02).
    */
    if exists (select *
                 from master.dbo.spt_values v, sysindexes i
                where i.status & v.number = v.number
                  and v.type = "I"
                  and v.number = 2
                  and i.id = object_id(@objname)
                  and i.indid = @indid)
    begin
      select @inddesc = @inddesc + ", " + v.name
        from master.dbo.spt_values v, sysindexes i
       where i.status & v.number = v.number
         and v.type = "I"
         and v.number = 2
         and i.id = object_id(@objname)
         and i.indid = @indid

      select @inddesc1 = @inddesc1 + " " + v.name
        from master.dbo.spt_values v, sysindexes i
       where i.status & v.number = v.number
         and v.type = "I"
         and v.number = 2
         and i.id = object_id(@objname)
         and i.indid = @indid
    end

    select @inddesc1 = @inddesc1 + @inddesc3
    /*
    **  See if the index is ignore_dupkey (0x01).
    */
    if exists (select *
                 from master.dbo.spt_values v, sysindexes i
                where i.status & v.number = v.number
                  and v.type = "I"
                  and v.number = 1
                  and i.id = object_id(@objname)
                  and i.indid = @indid)
    begin
      select @inddesc = @inddesc + ", " + v.name
        from master.dbo.spt_values v, sysindexes i
       where i.status & v.number = v.number
         and v.type = "I"
         and v.number = 1
         and i.id = object_id(@objname)
         and i.indid = @indid

      select @inddesc2 = " with " + v.name
        from master.dbo.spt_values v, sysindexes i
       where i.status & v.number = v.number
         and v.type = "I"
         and v.number = 1
         and i.id = object_id(@objname)
         and i.indid = @indid
    end

    /*
    **  See if the index is ignore_dup_row (0x04).
    */
    if exists (select *
                 from master.dbo.spt_values v, sysindexes i
                where i.status & v.number = v.number
                  and v.type = "I"
                  and v.number = 4
                  and i.id = object_id(@objname)
                  and i.indid = @indid)
    begin
      select @inddesc = @inddesc + ", " + v.name
        from master.dbo.spt_values v, sysindexes i
       where i.status & v.number = v.number
         and v.type = "I"
         and v.number = 4
         and i.id = object_id(@objname)
         and i.indid = @indid

      select @inddesc2 = " with " + v.name
        from master.dbo.spt_values v, sysindexes i
       where i.status & v.number = v.number
         and v.type = "I"
         and v.number = 4
         and i.id = object_id(@objname)
         and i.indid = @indid
    end

    /*
    **  See if the index is allow_dup_row (0x40).
    */
    if exists (select *
                 from master.dbo.spt_values v, sysindexes i
                where i.status & v.number = v.number
                  and v.type = "I"
                  and v.number = 64
                  and i.id = object_id(@objname)
                  and i.indid = @indid)
    begin
      select @inddesc = @inddesc + ", " + v.name   
        from master.dbo.spt_values v, sysindexes i
       where i.status & v.number = v.number
         and v.type = "I"
         and v.number = 64
         and i.id = object_id(@objname)
         and i.indid = @indid

      select @inddesc2 = " with " + v.name   
        from master.dbo.spt_values v, sysindexes i
       where i.status & v.number = v.number
         and v.type = "I"
         and v.number = 64
         and i.id = object_id(@objname)
         and i.indid = @indid
     end

    /*
    **  Add the location of the data.
    */
    select @inddesc = @inddesc + " located on " + s.name
      from syssegments s, sysindexes i
     where s.segment = i.segment
       and i.id = object_id(@objname)
       and i.indid = @indid

    select @inddesc4= " on " + s.name
      from syssegments s, sysindexes i
     where s.segment = i.segment
       and i.id = object_id(@objname)
       and i.indid = @indid
       and s.name not in ('default','system')

    /*
    **  Now we have the whole description for the index so we'll add
    **  the goods to the temporary table.
    */
    insert into #spindtab
    select @inddesc1 + name + ' on ' + @objname + '(',
           @inddesc2, 
           @inddesc4, 
           @keys
      from sysindexes
     where id = object_id(@objname)
       and indid = @indid
    /*
    **  Now move @indid to the next index.
    */
    select @lastindid = @indid
    select @indid = NULL
    select @indid = min(indid)
      from sysindexes
     where id = object_id(@objname)
       and indid > @lastindid
       and indid < 255
  end
  end
  else
  begin

  select @indid = min(indid)
    from sysindexes
   where id = object_id(@objname)
     and name = @indname
     and indid > 0
     and indid < 255

    select @inddesc1 = 'create '
    select @inddesc2 = ''
    select @inddesc4 = ''
  	
    select @keys = "", @i = 1

    set nocount on

    while @i <= 16
    begin
      select @thiskey = index_col(@objname, @indid, @i)

      if @thiskey = NULL
      begin
        goto keysdone2
      end

      if @i > 1
      begin
        select @keys = @keys + ", "
      end

      select @keys = @keys + index_col(@objname, @indid, @i)

      /*
      **  Increment @i so it will check for the next key.
      */
      select @i = @i + 1

    end


    /*
    **  When we get here we now have all the keys.

    */
    keysdone2:
      set nocount off

    /*
    **  Initialize the index description by figuring out if it's a
    **  clustered or nonclustered index.
    */
    if @indid = 1
    begin
      select @inddesc = "clustered"
      select @inddesc3 = " clustered index "
    end
    if @indid > 1
    begin
      select @inddesc3 = " nonclustered index "
      select @inddesc = "nonclustered"
    end

    /*
    **  Now we'll check out the status bits for this index and
    **  build an english description from them.
    */

    /*
    **  See if the index is unique (0x02).
    */
    if exists (select *
                 from master.dbo.spt_values v, sysindexes i
                where i.status & v.number = v.number
                  and v.type = "I"
                  and v.number = 2
                  and i.id = object_id(@objname)
                  and i.indid = @indid)
    begin
      select @inddesc = @inddesc + ", " + v.name
        from master.dbo.spt_values v, sysindexes i
       where i.status & v.number = v.number
         and v.type = "I"
         and v.number = 2
         and i.id = object_id(@objname)
         and i.indid = @indid

      select @inddesc1 = @inddesc1 + " " + v.name
        from master.dbo.spt_values v, sysindexes i
       where i.status & v.number = v.number
         and v.type = "I"
         and v.number = 2
         and i.id = object_id(@objname)
         and i.indid = @indid
    end

    select @inddesc1 = @inddesc1 + @inddesc3
    /*
    **  See if the index is ignore_dupkey (0x01).
    */
    if exists (select *
                 from master.dbo.spt_values v, sysindexes i
                where i.status & v.number = v.number
                  and v.type = "I"
                  and v.number = 1
                  and i.id = object_id(@objname)
                  and i.indid = @indid)
    begin
      select @inddesc = @inddesc + ", " + v.name
        from master.dbo.spt_values v, sysindexes i
       where i.status & v.number = v.number
         and v.type = "I"
         and v.number = 1
         and i.id = object_id(@objname)
         and i.indid = @indid

      select @inddesc2 = " with " + v.name
        from master.dbo.spt_values v, sysindexes i
       where i.status & v.number = v.number
         and v.type = "I"
         and v.number = 1
         and i.id = object_id(@objname)
         and i.indid = @indid
    end

    /*
    **  See if the index is ignore_dup_row (0x04).
    */
    if exists (select *
                 from master.dbo.spt_values v, sysindexes i
                where i.status & v.number = v.number
                  and v.type = "I"
                  and v.number = 4
                  and i.id = object_id(@objname)
                  and i.indid = @indid)
    begin
      select @inddesc = @inddesc + ", " + v.name
        from master.dbo.spt_values v, sysindexes i
       where i.status & v.number = v.number
         and v.type = "I"
         and v.number = 4
         and i.id = object_id(@objname)
         and i.indid = @indid

      select @inddesc2 = " with " + v.name
        from master.dbo.spt_values v, sysindexes i
       where i.status & v.number = v.number
         and v.type = "I"
         and v.number = 4
         and i.id = object_id(@objname)
         and i.indid = @indid
    end

    /*
    **  See if the index is allow_dup_row (0x40).
    */
    if exists (select *
                 from master.dbo.spt_values v, sysindexes i
                where i.status & v.number = v.number
                  and v.type = "I"
                  and v.number = 64
                  and i.id = object_id(@objname)
                  and i.indid = @indid)
    begin
      select @inddesc = @inddesc + ", " + v.name   
        from master.dbo.spt_values v, sysindexes i
       where i.status & v.number = v.number
         and v.type = "I"
         and v.number = 64
         and i.id = object_id(@objname)
         and i.indid = @indid

      select @inddesc2 = " with " + v.name   
        from master.dbo.spt_values v, sysindexes i
       where i.status & v.number = v.number
         and v.type = "I"
         and v.number = 64
         and i.id = object_id(@objname)
         and i.indid = @indid
     end

    /*
    **  Add the location of the data.
    */
    select @inddesc = @inddesc + " located on " + s.name
      from syssegments s, sysindexes i
     where s.segment = i.segment
       and i.id = object_id(@objname)
       and i.indid = @indid

    select @inddesc4= " on " + s.name
      from syssegments s, sysindexes i
     where s.segment = i.segment
       and i.id = object_id(@objname)
       and i.indid = @indid

    /*
    **  Now we have the whole description for the index so we'll add
    **  the goods to the temporary table.
    */
    insert into #spindtab
    select @inddesc1 + name + ' on ' + @objname,
           @inddesc2, 
           @inddesc4, 
           @keys
      from sysindexes
     where id = object_id(@objname)
       and indid = @indid
    /*
    **  Now move @indid to the next index.
    */
    select @lastindid = @indid
    select @indid = NULL
    select @indid = min(indid)
      from sysindexes
     where id = object_id(@objname)
       and indid > @lastindid
       and indid < 255

  end

/*
**  Now print out the contents of the temporary index table.
*/

select index_description1 = rtrim(index_description1),
       index_keys = index_keys,
       index_description2 = isnull(rtrim(index_description2),''),
       index_description3 = isnull(rtrim(index_description3),'')
from #spindtab

drop table #spindtab

end
go


create procedure sp_dbaman_get_user_type @type_name varchar(30)
as
begin

  declare @typeid smallint
  declare @storage_type varchar(35)
  declare @type_length  varchar(5)

  select @typeid = usertype

    from systypes

   where name = @type_name



  select @storage_type = st.name,

         @type_length = convert(varchar(5),s.length)

    from systypes s, systypes st

   where s.usertype = @typeid

     and s.type = st.type

     and st.name not in ("sysname", "nchar", "nvarchar")

     and st.usertype < 100


      if @storage_type like '%char%' 
         select @storage_type = @storage_type + '(' + @type_length + ')'
      select @storage_type
end
go

create procedure sp_dbaman_who @loginame varchar(30) = NULL as

declare @low int
declare @high int
declare @spidlow int
declare @spidhigh int
declare @msg varchar(250)
declare @len1 int, @len2 int

select @low = 0, @high = 32767, @spidlow = 0, @spidhigh = 32767

if @loginame is not NULL
begin
	select @low = suser_id(@loginame), @high = suser_id(@loginame)
	if @low is NULL
	begin
		if @loginame like "[0-9]%"
		begin
			select @spidlow = convert(int, @loginame),
			     @spidhigh = convert(int, @loginame),
			     @low = 0, @high = 32767
		end
		else
		begin
			/*
			** 17231, "No login with the specified name exists." 
			*/
			exec sp_getmessage 17231, @msg output
			print @msg
			return (1)
		end
	end
end

select @len1 = max(datalength(suser_name(suid))),
       @len2 = max(datalength(db_name(dbid)))
	from master..sysprocesses
	where suid >= @low and suid <= @high and
	      spid >= @spidlow and spid <= @spidhigh

if (@len1 > 12 or @len2 > 10)
    begin
	select spid,status,loginame=suser_name(suid),
	   hostname,blk=convert(char(3),blocked),
	   dbname=db_name(dbid),cmd
	from master..sysprocesses
	where suid >= @low and suid <= @high
	and spid >= @spidlow and spid <= @spidhigh
    end
else
    begin
	select spid,status,loginame=convert(char(12), suser_name(suid)),
	   hostname,blk=convert(char(3),blocked),
	   dbname=convert(char(10), db_name(dbid)),cmd
	from master..sysprocesses
	where suid >= @low and suid <= @high
	and spid >= @spidlow and spid <= @spidhigh
    end

return (0)


go